%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Fionnuala Connolly
%%% July 20th 2016
%%% Takes initial actuator geometry and material properties as input.
%%% Calculates F, where F(1) is an expression for the pressure, F(2) an
%%% expression for the axial force and F(3) an expression for the axial
%%% moment. These expressions are all Taylor expansions, as the full
%%% expressions are too complex.
%%% Solve F=0 to find x (extension, radius and twist per unit length)
%%% This function is for one family of fibers
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function F=numerics_Taylor(A,B,x,P,rho_elast_iso,rho_elast_aniso,alpha,rho,eps,eps2)

M2=cos(alpha);

C1=rho_elast_iso/2;

b = sqrt(x(2).^2+(1./x(1)).*(B^2-A^2));

W1t=C1.*(2.*A.^(-1).*(b.^2+(-1).*((-1).*A.^2+B.^2).*x(1).^(-1)).^(1/2)+( ...
  -2).*A.^3.*(b.^2+(-1).*((-1).*A.^2+B.^2).*x(1).^(-1)).^(-3/2).*x(1).^( ...
  -2));

W1tt=C1.*(2+6.*A.^4.*(b.^2+(-1).*((-1).*A.^2+B.^2).*x(1).^(-1)).^(-2).* ...
  x(1).^(-2));

W1ttt=(-24).*A.^5.*C1.*(b.^2+(-1).*((-1).*A.^2+B.^2).*x(1).^(-1)).^(-5/2) ...
  .*x(1).^(-2);

W2t=(2.*b.*B.^(-1)+(-2).*b.^(-3).*B.^3.*x(1).^(-2)).*(rho_elast_aniso/2)+2.* ...
  M2.*rho.*(b.*B.^(-1).*M2+b.*x(1).*(1+(-1).*M2.^2).^(1/2).*x(3)).*( ...
  x(1).^2.*(1+(-1).*M2.^2)+(b.*B.^(-1).*M2+b.*x(1).*(1+(-1).*M2.^2).^( ...
  1/2).*x(3)).^2).^(-1/2).*((-1)+(x(1).^2.*(1+(-1).*M2.^2)+(b.*B.^(-1) ...
  .*M2+b.*x(1).*(1+(-1).*M2.^2).^(1/2).*x(3)).^2).^(1/2));

W1tz=4.*A.^3.*C1.*(b.^2+(-1).*((-1).*A.^2+B.^2).*x(1).^(-1)).^(-3/2).* ...
  x(1).^(-3);

W1ttz=(-12).*A.^4.*C1.*(b.^2+(-1).*((-1).*A.^2+B.^2).*x(1).^(-1)).^(-2).* ...
  x(1).^(-3);

W1z=C1.*((-2).*A.^2.*(b.^2+(-1).*((-1).*A.^2+B.^2).*x(1).^(-1)).^(-1).* ...
  x(1).^(-3)+2.*x(1)+2.*(b.^2+(-1).*((-1).*A.^2+B.^2).*x(1).^(-1)).*x(1).* ...
  x(3).^2);

W2z=(rho_elast_aniso/2).*((-2).*b.^(-2).*B.^2.*x(1).^(-3)+2.*x(1)+2.*b.^2.*x(1).* ...
  x(3).^2)+rho.*(2.*x(1).*(1+(-1).*M2.^2)+2.*b.*(1+(-1).*M2.^2).^(1/2) ...
  .*x(3).*(b.*B.^(-1).*M2+b.*x(1).*(1+(-1).*M2.^2).^(1/2).*x(3))).*( ...
  x(1).^2.*(1+(-1).*M2.^2)+(b.*B.^(-1).*M2+b.*x(1).*(1+(-1).*M2.^2).^( ...
  1/2).*x(3)).^2).^(-1/2).*((-1)+(x(1).^2.*(1+(-1).*M2.^2)+(b.*B.^(-1) ...
  .*M2+b.*x(1).*(1+(-1).*M2.^2).^(1/2).*x(3)).^2).^(1/2));

W1ga=2.*C1.*(b.^2+(-1).*((-1).*A.^2+B.^2).*x(1).^(-1)).^(1/2).*x(1).^2.* ...
  x(3);

W2gb=2.*b.*x(1).^2.*(rho_elast_aniso/2).*x(3)+2.*x(1).*(1+(-1).*M2.^2).^(1/2).* ...
  rho.*(b.*B.^(-1).*M2+b.*x(1).*(1+(-1).*M2.^2).^(1/2).*x(3)).*(x(1).^2.* ...
  (1+(-1).*M2.^2)+(b.*B.^(-1).*M2+b.*x(1).*(1+(-1).*M2.^2).^(1/2).* ...
  x(3)).^2).^(-1/2).*((-1)+(x(1).^2.*(1+(-1).*M2.^2)+(b.*B.^(-1).*M2+ ...
  b.*x(1).*(1+(-1).*M2.^2).^(1/2).*x(3)).^2).^(1/2));


F=[x(2).^(-1).*A.*eps.*x(1).^(-1).*W1t+(1/2).*x(2).^(-3).*eps.^2.*x(1).^(-2).*( ...
  (-1).*A.^3.*W1t+x(2).*(A.^2+(-1).*x(2).^2.*x(1)).*W1tt)+(1/6).*x(2).^(-5).* ...
  A.^(-1).*eps.^3.*x(1).^(-3).*((3.*A.^6+(-1).*x(2).^2.*A.^4.*x(1)).*W1t+( ...
  -1).*x(2).*(A.^2+(-1).*x(2).^2.*x(1)).*((3.*A.^3+2.*x(2).^2.*A.*x(1)).*W1tt+x(2).* ...
  ((-1).*A.^2+x(2).^2.*x(1)).*W1ttt))+b.^(-1).*B.*eps2.*x(1).^(-1).*W2t-P;
  (1/6).*x(2).^(-3).*A.^(-1).*eps.^3.*x(1).^(-3).*((-1).*A.^2+x(2).^2.*x(1)).* ...
  pi.*((-1).*A.^4.*W1t+x(2).*((A.^3+(-2).*x(2).^2.*A.*x(1)).*W1tt+(-1).*( ...
  A.^2+(-1).*x(2).^2.*x(1)).*((-1).*x(2).*W1ttt+2.*A.*x(1).*W1ttz))+2.*A.^4.* ...
  x(1).*W1tz)+A.*eps.*x(1).^(-1).*pi.*((-1).*x(2).*W1t+2.*A.*x(1).*W1z)+(1/2) ...
  .*x(2).^(-1).*eps.^2.*x(1).^(-2).*pi.*((-1).*A.^3.*W1t+(A.^2+(-1).* ...
  x(2).^2.*x(1)).*((-1).*x(2).*W1tt+2.*A.*x(1).*W1tz)+2.*x(2).*A.^2.*x(1).^2.*W1z)+ ...
  B.*eps2.*x(1).^(-1).*pi.*((-1).*b.*W2t+2.*B.*x(1).*W2z);
  2.*x(2).*A.^2.*eps.*x(1).^(-1).*pi.*W1ga+x(2).^(-1).*A.*eps.^2.*x(1).^(-2).* ...
  pi.*((A.^3+x(2).^2.*A.*x(1)).*W1ga)+( ...
  1/3).*x(2).^(-3).*eps.^3.*x(1).^(-3).*pi.*(((-1).*A.^6+3.*x(2).^2.*A.^4.* ...
  x(1)).*W1ga)+2.*b.*B.^2.*eps2.*x(1).^(-1).*pi.*W2gb];
